import { useUser } from "@/stores/useUser";
import axios from "axios";
import { uniAdapter } from "fant-axios-adapter";

export default class ApiClient {
  public static server() {
    const BASE_URL = "https://ppmp.fenshenzhike.com/api";
    return ApiClient.create(BASE_URL);
  }

  public static create(baseUrl: string) {
    const instance = axios.create({
      withCredentials: true,
      baseURL: baseUrl,
      adapter: uniAdapter,
    });
    instance.interceptors.request.use(
      (request) => {
        if (request.headers) {
          request.headers.set(
            "Content-Type",
            "application/x-www-form-urlencoded"
          );
        } else {
          request.headers = new axios.AxiosHeaders();
          request.headers.set(
            "Content-Type",
            "application/x-www-form-urlencoded"
          );
        }
        request.headers.trace_id = new Date().getTime();
        return request;
      },
      (error) => Promise.reject(error)
    );
    instance.interceptors.response.use(
      (response) => {
        if (response.data.code === 10001) {
          const pages = getCurrentPages() as any[];
          const user = useUser();
          setTimeout(() => {
            uni.showToast({
              title: !!user.token ? "登录已过期，请重新登录" : "请先登录",
              icon: "none",
            });
            user.logout();
          }, 300);
          if (
            !pages[pages.length - 1].$page ||
            (pages[pages.length - 1].$page &&
              pages[pages.length - 1].$page.fullPath !== "/pages/login/index")
          ) {
            uni.reLaunch({ url: "/pages/login/index" });
          }
        }
        if (!response.data.code || response.data.code === 10000) {
          return response;
        } else {
          const error: Record<string, any> = {};
          if (response.data.code) {
            error.code = response.data.code;
          }
          if (response.data.message) {
            error.message = response.data.message;
          } else {
            error.message = `服务器内部错误:${response.status}`;
          }
          // error.response = response.data;
          return Promise.reject(error);
        }
      },
      (error) => {
        if (error.status !== 0 && !error.status) {
          const newError = error as any;
          newError.msg = newError.errMsg || "网络错误";
          return Promise.reject(newError);
        }
        const pages = getCurrentPages() as any[];
        const user = useUser();
        switch (error.status) {
          case 1:
            error.msg = "网络超时";
            break;
          case 401:
            user.logout();
            error.msg = "请先登录";
            setTimeout(() => {
              uni.showToast({ title: "登录已过期，请重新登录", icon: "none" });
            }, 300);
            if (
              !pages[pages.length - 1].$page ||
              (pages[pages.length - 1].$page &&
                pages[pages.length - 1].$page.fullPath !== "/pages/login/index")
            ) {
              uni.reLaunch({ url: "/pages/login/index" });
            }
            break;
          case 403:
            error.msg = `${error.status} 禁止访问!`;
            break;
          case 500:
            error.msg = `${error.status} 服务内部异常!`;
            break;
          case 502:
            error.msg = `${error.status} 服务器暂不可用!`;
            break;
          case 503:
            error.msg = `${error.status} 服务器升级中!`;
            break;
          case 404:
            error.msg = `${error.status} 服务器无回应!`;
            break;
          default:
            error.msg = `${error.status} 未知错误!`;
        }
        return Promise.reject(error);
      }
    );
    return instance;
  }
}
